@@ -5,7 +5,8 @@
  *
  * Authored By Matthew Allum  <mallum@openedhand.com>
  *
- * Copyright (C) 2006 OpenedHand
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd
+ * Copyright (C) 2009, 2010 Intel Corp
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,11 +31,13 @@
 
 /* clutter-actor.h */
 
-#include <glib-object.h>
 #include <pango/pango.h>
+#include <atk/atk.h>
+
+#include <clutter/clutter-types.h>
 
 #include <clutter/clutter-color.h>
-#include <clutter/clutter-types.h>
+#include <clutter/clutter-effect.h>
 #include <clutter/clutter-event.h>
 #include <clutter/clutter-shader.h>
 
@@ -133,6 +136,25 @@ typedef enum
   CLUTTER_ABSOLUTE_ORIGIN_CHANGED = 1 << 1
 } ClutterAllocationFlags;
 
+/**
+ * ClutterRedrawFlags:
+ * @CLUTTER_REDRAW_CLIPPED_TO_BOX: Tells clutter the redraw is clipped
+ *   to a given clip box in actor coordinates.
+ * @CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION: Tells clutter the maximum
+ *   extents of what needs to be redrawn lies within the actors
+ *   current allocation.
+ *
+ * Flags passed to the clutter_actor_queue_redraw_with_clip ()
+ * function
+ *
+ * Since: 1.2
+ */
+typedef enum
+{
+  CLUTTER_REDRAW_CLIPPED_TO_BOX         = 0,
+  CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION  = 1 << 1
+} ClutterRedrawFlags;
+
 /**
  * ClutterActor:
  * @flags: #ClutterActorFlags
@@ -188,6 +210,8 @@ struct _ClutterActor
  * @apply_transform: virtual function, used when applying the transformations
  *   to an actor before painting it or when transforming coordinates or
  *   the allocation; it must chain up to the parent's implementation
+ * @get_accessible: virtual function, returns the accessible object that
+ *   describes the actor to an assistive technology.
  * @parent_set: signal class handler for the #ClutterActor::parent-set
  * @destroy: signal class handler for #ClutterActor::destroy
  * @pick: virtual function, used to draw an outline of the actor with
@@ -279,9 +303,12 @@ struct _ClutterActorClass
 
   void     (* queue_relayout)       (ClutterActor         *actor);
 
+  /* accessibility support */
+  AtkObject * (* get_accessible)    (ClutterActor         *actor);
+
   /*< private >*/
   /* padding for future expansion */
-  gpointer _padding_dummy[31];
+  gpointer _padding_dummy[30];
 };
 
 GType                 clutter_actor_get_type                  (void) G_GNUC_CONST;
@@ -302,6 +329,7 @@ void                  clutter_actor_map                       (ClutterActor
 void                  clutter_actor_unmap                     (ClutterActor          *self);
 void                  clutter_actor_paint                     (ClutterActor          *self);
 void                  clutter_actor_queue_redraw              (ClutterActor          *self);
+
 void                  clutter_actor_queue_relayout            (ClutterActor          *self);
 void                  clutter_actor_destroy                   (ClutterActor          *self);
 
@@ -333,6 +361,13 @@ void                  clutter_actor_allocate_available_size   (ClutterActor
                                                                gfloat                 available_width,
                                                                gfloat                 available_height,
                                                                ClutterAllocationFlags flags);
+void                  clutter_actor_allocate_align_fill       (ClutterActor          *self,
+                                                               const ClutterActorBox *box,
+                                                               gdouble                x_align,
+                                                               gdouble                y_align,
+                                                               gboolean               x_fill,
+                                                               gboolean               y_fill,
+                                                               ClutterAllocationFlags flags);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
@@ -420,6 +455,9 @@ void                  clutter_actor_get_clip                  (ClutterActor
                                                                gfloat                *yoff,
                                                                gfloat                *width,
                                                                gfloat                *height);
+void                  clutter_actor_set_clip_to_allocation    (ClutterActor          *self,
+                                                               gboolean               clip_set);
+gboolean              clutter_actor_get_clip_to_allocation    (ClutterActor          *self);
 
 void                  clutter_actor_set_parent                (ClutterActor          *self,
                                                                ClutterActor          *parent);
@@ -427,6 +465,8 @@ ClutterActor *        clutter_actor_get_parent                (ClutterActor
 void                  clutter_actor_reparent                  (ClutterActor          *self,
                                                                ClutterActor          *new_parent);
 void                  clutter_actor_unparent                  (ClutterActor          *self);
+gboolean              clutter_actor_contains                  (ClutterActor          *self,
+							       ClutterActor          *descendant);
 ClutterActor*         clutter_actor_get_stage                 (ClutterActor          *actor);
 
 void                  clutter_actor_raise                     (ClutterActor          *self,
@@ -532,13 +572,18 @@ void clutter_actor_get_transformation_matrix          (ClutterActor        *self
                                                        CoglMatrix          *matrix);
 
 gboolean clutter_actor_is_in_clone_paint              (ClutterActor        *self);
+gboolean clutter_actor_has_pointer                    (ClutterActor        *self);
 
 void                 clutter_actor_set_text_direction (ClutterActor         *self,
                                                        ClutterTextDirection  text_dir);
 ClutterTextDirection clutter_actor_get_text_direction (ClutterActor         *self);
 
-void                 clutter_actor_push_internal      (void);
-void                 clutter_actor_pop_internal       (void);
+void                 clutter_actor_push_internal      (ClutterActor         *self);
+void                 clutter_actor_pop_internal       (ClutterActor         *self);
+
+gboolean             clutter_actor_has_allocation     (ClutterActor         *self);
+
+AtkObject *          clutter_actor_get_accessible     (ClutterActor         *self);
 
 G_END_DECLS
 
