@@ -145,6 +145,7 @@
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
+#include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-scriptable.h"
 #include "clutter-script-private.h"
@@ -158,9 +159,13 @@ enum
   PROP_DURATION,
   PROP_LOOP,
   PROP_TIMELINE,
-  PROP_ALPHA
+  PROP_ALPHA,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   STARTED,
@@ -212,6 +217,7 @@ static void
 clutter_animation_real_completed (ClutterAnimation *self)
 {
   ClutterAnimationPrivate *priv = self->priv;
+  ClutterAnimatable *animatable = NULL;
   ClutterAnimation *animation;
   ClutterTimeline *timeline;
   ClutterTimelineDirection direction;
@@ -221,6 +227,9 @@ clutter_animation_real_completed (ClutterAnimation *self)
   timeline = clutter_animation_get_timeline (self);
   direction = clutter_timeline_get_direction (timeline);
 
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    animatable = CLUTTER_ANIMATABLE (priv->object);
+
   /* explicitly set the final state of the animation */
   CLUTTER_NOTE (ANIMATION, "Set final state on object [%p]", priv->object);
   g_hash_table_iter_init (&iter, priv->properties);
@@ -235,7 +244,10 @@ clutter_animation_real_completed (ClutterAnimation *self)
       else
         p_value = clutter_interval_peek_initial_value (interval);
 
-      g_object_set_property (priv->object, p_name, p_value);
+      if (animatable != NULL)
+        clutter_animatable_set_final_state (animatable, p_name, p_value);
+      else
+        g_object_set_property (priv->object, p_name, p_value);
     }
 
   /* at this point, if this animation was created by clutter_actor_animate()
@@ -448,10 +460,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("object",
-                               "Object",
-                               "Object to which the animation applies",
+                               P_("Object"),
+                               P_("Object to which the animation applies"),
                                G_TYPE_OBJECT,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_OBJECT] = pspec;
   g_object_class_install_property (gobject_class, PROP_OBJECT, pspec);
 
   /**
@@ -464,11 +477,12 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_ulong ("mode",
-                              "Mode",
-                              "The mode of the animation",
+                              P_("Mode"),
+                              P_("The mode of the animation"),
                               0, G_MAXULONG,
                               CLUTTER_LINEAR,
                               CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_MODE] = pspec;
   g_object_class_install_property (gobject_class, PROP_MODE, pspec);
 
   /**
@@ -479,10 +493,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_uint ("duration",
-                             "Duration",
-                             "Duration of the animation, in milliseconds",
+                             P_("Duration"),
+                             P_("Duration of the animation, in milliseconds"),
                              0, G_MAXUINT, 0,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 
   /**
@@ -493,10 +508,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_boolean ("loop",
-                                "Loop",
-                                "Whether the animation should loop",
+                                P_("Loop"),
+                                P_("Whether the animation should loop"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_LOOP] = pspec;
   g_object_class_install_property (gobject_class, PROP_LOOP, pspec);
 
   /**
@@ -507,10 +523,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("timeline",
-                               "Timeline",
-                               "The timeline used by the animation",
+                               P_("Timeline"),
+                               P_("The timeline used by the animation"),
                                CLUTTER_TYPE_TIMELINE,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_TIMELINE] = pspec;
   g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
 
   /**
@@ -521,10 +538,11 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_object ("alpha",
-                               "Alpha",
-                               "The alpha used by the animation",
+                               P_("Alpha"),
+                               P_("The alpha used by the animation"),
                                CLUTTER_TYPE_ALPHA,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ALPHA] = pspec;
   g_object_class_install_property (gobject_class, PROP_ALPHA, pspec);
 
   /**
@@ -542,7 +560,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, started),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -563,7 +581,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, completed),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
@@ -580,6 +598,7 @@ clutter_animation_init (ClutterAnimation *self)
 
 static inline void
 clutter_animation_bind_property_internal (ClutterAnimation *animation,
+                                          const gchar      *property_name,
                                           GParamSpec       *pspec,
                                           ClutterInterval  *interval)
 {
@@ -589,17 +608,18 @@ clutter_animation_bind_property_internal (ClutterAnimation *animation,
     {
       g_warning ("Cannot bind property '%s': the interval is out "
                  "of bounds",
-                 pspec->name);
+                 property_name);
       return;
     }
 
   g_hash_table_insert (priv->properties,
-                       g_strdup (pspec->name),
+                       g_strdup (property_name),
                        g_object_ref_sink (interval));
 }
 
 static inline void
 clutter_animation_update_property_internal (ClutterAnimation *animation,
+                                            const gchar      *property_name,
                                             GParamSpec       *pspec,
                                             ClutterInterval  *interval)
 {
@@ -609,12 +629,12 @@ clutter_animation_update_property_internal (ClutterAnimation *animation,
     {
       g_warning ("Cannot bind property '%s': the interval is out "
                  "of bounds",
-                 pspec->name);
+                 property_name);
       return;
     }
 
   g_hash_table_replace (priv->properties,
-                        g_strdup (pspec->name),
+                        g_strdup (property_name),
                         g_object_ref_sink (interval));
 }
 
@@ -624,7 +644,6 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
                                  GType             argtype)
 {
   ClutterAnimationPrivate *priv;
-  GObjectClass *klass;
   GParamSpec *pspec;
   GType pspec_type;
 
@@ -647,9 +666,20 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
       return NULL;
     }
 
-  klass = G_OBJECT_GET_CLASS (priv->object);
-  pspec = g_object_class_find_property (klass, property_name);
-  if (!pspec)
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    {
+      ClutterAnimatable *animatable = CLUTTER_ANIMATABLE (priv->object);
+
+      pspec = clutter_animatable_find_property (animatable, property_name);
+    }
+  else
+    {
+      GObjectClass *klass = G_OBJECT_GET_CLASS (priv->object);
+
+      pspec = g_object_class_find_property (klass, property_name);
+    }
+
+  if (pspec == NULL)
     {
       g_warning ("Cannot bind property '%s': objects of type '%s' have "
                  "no such property",
@@ -715,7 +745,9 @@ clutter_animation_bind_interval (ClutterAnimation *animation,
   if (pspec == NULL)
     return NULL;
 
-  clutter_animation_bind_property_internal (animation, pspec, interval);
+  clutter_animation_bind_property_internal (animation, property_name,
+                                            pspec,
+                                            interval);
 
   return animation;
 }
@@ -760,12 +792,20 @@ clutter_animation_bind (ClutterAnimation *animation,
     return NULL;
 
   g_value_init (&initial, G_PARAM_SPEC_VALUE_TYPE (pspec));
-  g_object_get_property (priv->object, property_name, &initial);
+
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    clutter_animatable_get_initial_state (CLUTTER_ANIMATABLE (priv->object),
+                                          property_name,
+                                          &initial);
+  else
+    g_object_get_property (priv->object, property_name, &initial);
 
   interval = clutter_interval_new_with_values (type, &initial, final);
   g_value_unset (&initial);
 
-  clutter_animation_bind_property_internal (animation, pspec, interval);
+  clutter_animation_bind_property_internal (animation, property_name,
+                                            pspec,
+                                            interval);
 
   return animation;
 }
@@ -845,7 +885,6 @@ clutter_animation_update_interval (ClutterAnimation *animation,
                                    ClutterInterval  *interval)
 {
   ClutterAnimationPrivate *priv;
-  GObjectClass *klass;
   GParamSpec *pspec;
   GType pspec_type, int_type;
 
@@ -863,9 +902,20 @@ clutter_animation_update_interval (ClutterAnimation *animation,
       return;
     }
 
-  klass = G_OBJECT_GET_CLASS (priv->object);
-  pspec = g_object_class_find_property (klass, property_name);
-  if (!pspec)
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    {
+      ClutterAnimatable *animatable = CLUTTER_ANIMATABLE (priv->object);
+
+      pspec = clutter_animatable_find_property (animatable, property_name);
+    }
+  else
+    {
+      GObjectClass *klass = G_OBJECT_GET_CLASS (priv->object);
+
+      pspec = g_object_class_find_property (klass, property_name);
+    }
+
+  if (pspec == NULL)
     {
       g_warning ("Cannot update property '%s': objects of type '%s' have "
                  "no such property",
@@ -889,7 +939,9 @@ clutter_animation_update_interval (ClutterAnimation *animation,
       return;
     }
 
-  clutter_animation_update_property_internal (animation, pspec, interval);
+  clutter_animation_update_property_internal (animation, property_name,
+                                              pspec,
+                                              interval);
 }
 
 /**
@@ -1050,7 +1102,12 @@ on_alpha_notify (GObject          *gobject,
         }
 
       if (apply)
-        g_object_set_property (priv->object, p_name, &value);
+        {
+          if (is_animatable)
+            clutter_animatable_set_final_state (animatable, p_name, &value);
+          else
+            g_object_set_property (priv->object, p_name, &value);
+        }
 
       g_value_unset (&value);
     }
@@ -1081,7 +1138,7 @@ clutter_animation_get_alpha_internal (ClutterAnimation *animation)
 
       priv->alpha = g_object_ref_sink (alpha);
 
-      g_object_notify (G_OBJECT (animation), "alpha");
+      _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_ALPHA]);
     }
 
   return priv->alpha;
@@ -1116,7 +1173,7 @@ clutter_animation_get_timeline_internal (ClutterAnimation *animation)
   /* the alpha owns the timeline now */
   g_object_unref (timeline);
 
-  g_object_notify (G_OBJECT (animation), "timeline");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
 
   return timeline;
 }
@@ -1181,7 +1238,7 @@ clutter_animation_set_object (ClutterAnimation *animation,
   if (object != NULL)
     priv->object = g_object_ref (object);
 
-  g_object_notify (G_OBJECT (animation), "object");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_OBJECT]);
 }
 
 /**
@@ -1230,7 +1287,7 @@ clutter_animation_set_mode (ClutterAnimation *animation,
 
   clutter_alpha_set_mode (alpha, mode);
 
-  g_object_notify (G_OBJECT (animation), "mode");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_MODE]);
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1272,7 +1329,7 @@ clutter_animation_get_mode (ClutterAnimation *animation)
  */
 void
 clutter_animation_set_duration (ClutterAnimation *animation,
-                                gint              msecs)
+                                guint             msecs)
 {
   ClutterTimeline *timeline;
 
@@ -1286,7 +1343,7 @@ clutter_animation_set_duration (ClutterAnimation *animation,
   clutter_timeline_set_duration (timeline, msecs);
   clutter_timeline_rewind (timeline);
 
-  g_object_notify (G_OBJECT (animation), "duration");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1319,7 +1376,7 @@ clutter_animation_set_loop (ClutterAnimation *animation,
   timeline = clutter_animation_get_timeline_internal (animation);
   clutter_timeline_set_loop (timeline, loop);
 
-  g_object_notify (G_OBJECT (animation), "loop");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1412,9 +1469,9 @@ clutter_animation_set_timeline (ClutterAnimation *animation,
 
   alpha = clutter_animation_get_alpha_internal (animation);
   clutter_alpha_set_timeline (alpha, timeline);
-  g_object_notify (G_OBJECT (animation), "timeline");
-  g_object_notify (G_OBJECT (animation), "duration");
-  g_object_notify (G_OBJECT (animation), "loop");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
 
   if (timeline)
     {
@@ -1542,11 +1599,11 @@ clutter_animation_set_alpha (ClutterAnimation *animation,
 
 out:
   /* emit all relevant notifications */
-  g_object_notify (G_OBJECT (animation), "mode");
-  g_object_notify (G_OBJECT (animation), "duration");
-  g_object_notify (G_OBJECT (animation), "loop");
-  g_object_notify (G_OBJECT (animation), "alpha");
-  g_object_notify (G_OBJECT (animation), "timeline");
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_MODE]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_DURATION]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_LOOP]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_ALPHA]);
+  _clutter_notify_by_pspec (G_OBJECT (animation), obj_props[PROP_TIMELINE]);
 
   g_object_thaw_notify (G_OBJECT (animation));
 }
@@ -1688,26 +1745,39 @@ done:
       GValue cur_value = { 0, };
 
       g_value_init (&cur_value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      g_object_get_property (priv->object, property_name, &cur_value);
+
+      if (CLUTTER_IS_ANIMATABLE (priv->object))
+        clutter_animatable_get_initial_state (CLUTTER_ANIMATABLE (priv->object),
+                                              property_name,
+                                              &cur_value);
+      else
+        g_object_get_property (priv->object, property_name, &cur_value);
 
       interval =
         clutter_interval_new_with_values (G_PARAM_SPEC_VALUE_TYPE (pspec),
                                           &cur_value,
                                           &real_value);
 
-      if (!clutter_animation_has_property (animation, pspec->name))
-        clutter_animation_bind_property_internal (animation,
+      if (!clutter_animation_has_property (animation, property_name))
+        clutter_animation_bind_property_internal (animation, property_name,
                                                   pspec,
                                                   interval);
       else
-        clutter_animation_update_property_internal (animation,
+        clutter_animation_update_property_internal (animation, property_name,
                                                     pspec,
                                                     interval);
 
       g_value_unset (&cur_value);
     }
   else
-    g_object_set_property (priv->object, property_name, &real_value);
+    {
+      if (CLUTTER_IS_ANIMATABLE (priv->object))
+        clutter_animatable_set_final_state (CLUTTER_ANIMATABLE (priv->object),
+                                            property_name,
+                                            &real_value);
+      else
+        g_object_set_property (priv->object, property_name, &real_value);
+    }
 
   g_value_unset (&real_value);
 }
@@ -1719,16 +1789,20 @@ clutter_animation_setupv (ClutterAnimation    *animation,
                           const GValue        *values)
 {
   ClutterAnimationPrivate *priv = animation->priv;
-  GObjectClass *klass;
+  ClutterAnimatable *animatable = NULL;
+  GObjectClass *klass = NULL;
   gint i;
-  gboolean is_fixed = FALSE;
 
-  klass = G_OBJECT_GET_CLASS (priv->object);
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    animatable = CLUTTER_ANIMATABLE (priv->object);
+  else
+    klass = G_OBJECT_GET_CLASS (priv->object);
 
   for (i = 0; i < n_properties; i++)
     {
       const gchar *property_name = properties[i];
       GParamSpec *pspec;
+      gboolean is_fixed = FALSE;
 
       if (g_str_has_prefix (property_name, "fixed::"))
         {
@@ -1736,8 +1810,12 @@ clutter_animation_setupv (ClutterAnimation    *animation,
           is_fixed = TRUE;
         }
 
-      pspec = g_object_class_find_property (klass, property_name);
-      if (!pspec)
+      if (animatable != NULL)
+        pspec = clutter_animatable_find_property (animatable, property_name);
+      else
+        pspec = g_object_class_find_property (klass, property_name);
+
+      if (pspec == NULL)
         {
           g_warning ("Cannot bind property '%s': objects of type '%s' do "
                      "not have this property",
@@ -1792,10 +1870,14 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
                                 va_list           var_args)
 {
   ClutterAnimationPrivate *priv = animation->priv;
-  GObjectClass *klass;
+  ClutterAnimatable *animatable = NULL;
+  GObjectClass *klass = NULL;
   const gchar *property_name;
 
-  klass = G_OBJECT_GET_CLASS (priv->object);
+  if (CLUTTER_IS_ANIMATABLE (priv->object))
+    animatable = CLUTTER_ANIMATABLE (priv->object);
+  else
+    klass = G_OBJECT_GET_CLASS (priv->object);
 
   property_name = first_property_name;
   while (property_name != NULL)
@@ -1827,8 +1909,13 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
               is_fixed = TRUE;
             }
 
-          pspec = g_object_class_find_property (klass, property_name);
-          if (!pspec)
+          if (animatable != NULL)
+            pspec = clutter_animatable_find_property (animatable,
+                                                      property_name);
+          else
+            pspec = g_object_class_find_property (klass, property_name);
+
+          if (pspec == NULL)
             {
               g_warning ("Cannot bind property '%s': objects of type '%s' do "
                          "not have this property",
@@ -1837,8 +1924,16 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
               break;
             }
 
+#if GLIB_CHECK_VERSION (2, 23, 2)
+          G_VALUE_COLLECT_INIT (&final, G_PARAM_SPEC_VALUE_TYPE (pspec),
+                                var_args, 0,
+                                &error);
+#else
+          /* this is the same as G_VALUE_COLLECT_INIT(), but slower */
           g_value_init (&final, G_PARAM_SPEC_VALUE_TYPE (pspec));
           G_VALUE_COLLECT (&final, var_args, 0, &error);
+#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
+
           if (error)
             {
               g_warning ("%s: %s", G_STRLOC, error);
@@ -1891,7 +1986,7 @@ animation_create_for_actor (ClutterActor *actor)
  * @actor: a #ClutterActor
  * @alpha: a #ClutterAlpha
  * @first_property_name: the name of a property
- * @VarArgs: a %NULL terminated list of property names and
+ * @Varargs: a %NULL terminated list of property names and
  *   property values
  *
  * Animates the given list of properties of @actor between the current
@@ -1948,7 +2043,7 @@ clutter_actor_animate_with_alpha (ClutterActor *actor,
  * @mode: an animation mode logical id
  * @timeline: a #ClutterTimeline
  * @first_property_name: the name of a property
- * @VarArgs: a %NULL terminated list of property names and
+ * @Varargs: a %NULL terminated list of property names and
  *   property values
  *
  * Animates the given list of properties of @actor between the current
@@ -1999,7 +2094,7 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  * @mode: an animation mode logical id
  * @duration: duration of the animation, in milliseconds
  * @first_property_name: the name of a property
- * @VarArgs: a %NULL terminated list of property names and
+ * @Varargs: a %NULL terminated list of property names and
  *   property values
  *
  * Animates the given list of properties of @actor between the current
@@ -2187,7 +2282,7 @@ clutter_actor_animate (ClutterActor *actor,
  * @n_properties: number of property names and values
  * @properties: (array length=n_properties) (element-type utf8): a vector
  *    containing the property names to set
- * @values: (array length=n_properies): a vector containing the
+ * @values: (array length=n_properties): a vector containing the
  *    property values to set
  *
  * Animates the given list of properties of @actor between the current
@@ -2239,7 +2334,7 @@ clutter_actor_animatev (ClutterActor        *actor,
  * @n_properties: number of property names and values
  * @properties: (array length=n_properties) (element-type utf8): a vector
  *    containing the property names to set
- * @values: (array length=n_properies): a vector containing the
+ * @values: (array length=n_properties): a vector containing the
  *    property values to set
  *
  * Animates the given list of properties of @actor between the current
@@ -2294,7 +2389,7 @@ clutter_actor_animate_with_timelinev (ClutterActor        *actor,
  * @n_properties: number of property names and values
  * @properties: (array length=n_properties) (element-type utf8): a vector
  *    containing the property names to set
- * @values: (array length=n_properies): a vector containing the
+ * @values: (array length=n_properties): a vector containing the
  *    property values to set
  *
  * Animates the given list of properties of @actor between the current
