@@ -18,9 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -80,9 +78,13 @@ enum
   PROP_DIRECTION,
   PROP_CENTER_X,
   PROP_CENTER_Y,
-  PROP_CENTER_Z
+  PROP_CENTER_Z,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 typedef struct {
   gdouble angle;
 } RotateFrameClosure;
@@ -272,11 +274,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-start",
-                               "Angle Begin",
-                               "Initial angle",
+                               P_("Angle Begin"),
+                               P_("Initial angle"),
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_START] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_START,
                                    pspec);
@@ -289,11 +292,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-end",
-                               "Angle End",
-                               "Final angle",
+                               P_("Angle End"),
+                               P_("Final angle"),
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_END] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_END,
                                    pspec);
@@ -306,11 +310,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_enum ("axis",
-                             "Axis",
-                             "Axis of rotation",
+                             P_("Axis"),
+                             P_("Axis of rotation"),
                              CLUTTER_TYPE_ROTATE_AXIS,
                              CLUTTER_Z_AXIS,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_AXIS] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_AXIS,
                                    pspec);
@@ -323,11 +328,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_enum ("direction",
-                             "Direction",
-                             "Direction of rotation",
+                             P_("Direction"),
+                             P_("Direction of rotation"),
                              CLUTTER_TYPE_ROTATE_DIRECTION,
                              CLUTTER_ROTATE_CW,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DIRECTION] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_DIRECTION,
                                    pspec);
@@ -340,11 +346,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-x",
-                            "Center-X",
-                            "X center of rotation",
+                            P_("Center X"),
+                            P_("X coordinate of the center of rotation"),
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_X] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_X,
                                    pspec);
@@ -357,11 +364,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-y",
-                            "Center-Y",
-                            "Y center of rotation",
+                            P_("Center Y"),
+                            P_("Y coordinate of the center of rotation"),
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_Y] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Y,
                                    pspec);
@@ -374,11 +382,12 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("center-z",
-                            "Center-Z",
-                            "Z center of rotation",
+                            P_("Center Z"),
+                            P_("Z coordinate of the center of rotation"),
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_Z] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Z,
                                    pspec);
@@ -477,7 +486,7 @@ clutter_behaviour_rotate_set_axis (ClutterBehaviourRotate *rotate,
     {
       priv->axis = axis;
 
-      g_object_notify (G_OBJECT (rotate), "axis");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_AXIS]);
     }
 }
 
@@ -523,7 +532,7 @@ clutter_behaviour_rotate_set_direction (ClutterBehaviourRotate *rotate,
     {
       priv->direction = direction;
 
-      g_object_notify (G_OBJECT (rotate), "direction");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_DIRECTION]);
     }
 }
 
@@ -583,14 +592,14 @@ clutter_behaviour_rotate_set_bounds (ClutterBehaviourRotate *rotate,
     {
       priv->angle_start = clamp_angle (angle_start);
 
-      g_object_notify (G_OBJECT (rotate), "angle-start");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_START]);
     }
 
   if (priv->angle_end != angle_end)
     {
       priv->angle_end = clamp_angle (angle_end);
 
-      g_object_notify (G_OBJECT (rotate), "angle-end");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_END]);
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
@@ -625,19 +634,19 @@ clutter_behaviour_rotate_set_center (ClutterBehaviourRotate *rotate,
   if (priv->center_x != x)
     {
       priv->center_x = x;
-      g_object_notify (G_OBJECT (rotate), "center-x");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_X]);
     }
 
   if (priv->center_y != y)
     {
       priv->center_y = y;
-      g_object_notify (G_OBJECT (rotate), "center-y");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Y]);
     }
 
   if (priv->center_z != z)
     {
       priv->center_z = z;
-      g_object_notify (G_OBJECT (rotate), "center-z");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Z]);
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
