@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -47,7 +47,7 @@
  *
  * A simple object might be defined as:
  *
- * |[
+ * <informalexample><programlisting><![CDATA[
  * {
  *   "id"     : "red-button",
  *   "type"   : "ClutterRectangle",
@@ -55,7 +55,7 @@
  *   "height" : 100,
  *   "color"  : "&num;ff0000ff"
  * }
- * ]|
+ * ]]></programlisting></informalexample>
  *
  * This will produce a red #ClutterRectangle, 100x100 pixels wide, and
  * with a ClutterScript id of "red-button"; it can be retrieved by calling:
@@ -78,7 +78,7 @@
  * Behaviours and timelines can also be defined inside a UI definition
  * buffer:
  *
- * |[
+ * <informalexample><programlisting><![CDATA[
  * {
  *   "id"          : "rotate-behaviour",
  *   "type"        : "ClutterBehaviourRotate",
@@ -90,25 +90,25 @@
  *     "mode"     : "easeInSine"
  *   }
  * }
- * ]|
+ * ]]></programlisting></informalexample>
  *
  * And then to apply a defined behaviour to an actor defined inside the
  * definition of an actor, the "behaviour" member can be used:
  *
- * |[
+ * <informalexample><programlisting><![CDATA[
  * {
  *   "id" : "my-rotating-actor",
  *   "type" : "ClutterTexture",
  *   ...
  *   "behaviours" : [ "rotate-behaviour" ]
  * }
- * ]|
+ * ]]></programlisting></informalexample>
  *
  * A #ClutterAlpha belonging to a #ClutterBehaviour can only be defined
  * implicitly like in the example above, or explicitly by setting the
  * "alpha" property to point to a previously defined #ClutterAlpha, e.g.:
  *
- * |[
+ * <informalexample><programlisting><![CDATA[
  * {
  *   "id"          : "rotate-behaviour",
  *   "type"        : "ClutterBehaviourRotate",
@@ -127,7 +127,7 @@
  *     "function" : "custom_sine_alpha"
  *   }
  * }
- * ]|
+ * ]]></programlisting></informalexample>
  *
  * Implicitely defined #ClutterAlpha<!-- -->s and #ClutterTimeline<!-- -->s
  * can omit the <varname>id</varname> member, as well as the
@@ -139,7 +139,7 @@
  * then autoconnected to their respective signals using the
  * clutter_script_connect_signals() function:
  *
- * |[
+ * <informalexample><programlisting><![CDATA[
  *   ...
  *   "signals" : [
  *     { "name" : "button-press-event", "handler" : "on_button_press" },
@@ -150,7 +150,7 @@
  *     },
  *   ],
  *   ...
- * ]|
+ * ]]></programlisting></informalexample>
  *
  * Signal handler definitions must have a "name" and a "handler" members;
  * they can also have the "after" and "swapped" boolean members (for the
@@ -217,9 +217,13 @@ enum
   PROP_0,
 
   PROP_FILENAME_SET,
-  PROP_FILENAME
+  PROP_FILENAME,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 #define CLUTTER_SCRIPT_GET_PRIVATE(obj) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_SCRIPT, ClutterScriptPrivate))
 
@@ -308,9 +312,6 @@ object_info_free (gpointer data)
       g_list_foreach (oinfo->children, (GFunc) g_free, NULL);
       g_list_free (oinfo->children);
 
-      g_list_foreach (oinfo->behaviours, (GFunc) g_free, NULL);
-      g_list_free (oinfo->behaviours);
-
       /* we unref top-level objects and leave the actors alone,
        * unless we are unmerging in which case we have to destroy
        * the actor to unparent them
@@ -400,10 +401,11 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_boolean ("filename-set",
-                                "Filename Set",
-                                "Whether the :filename property is set",
+                                P_("Filename Set"),
+                                P_("Whether the :filename property is set"),
                                 FALSE,
                                 CLUTTER_PARAM_READABLE);
+  obj_props[PROP_FILENAME_SET] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME_SET, pspec);
 
   /**
@@ -415,10 +417,11 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_string ("filename",
-                               "Filename",
-                               "The path of the currently parsed file",
+                               P_("Filename"),
+                               P_("The path of the currently parsed file"),
                                NULL,
                                CLUTTER_PARAM_READABLE);
+  obj_props[PROP_FILENAME] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME, pspec);
 }
 
@@ -1091,8 +1094,8 @@ clutter_script_lookup_filename (ClutterScript *script,
   gchar *dirname;
   gchar *retval;
 
-  g_return_val_if_fail (CLUTTER_IS_SCRIPT (script), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (CLUTTER_IS_SCRIPT (script), NULL);
+  g_return_val_if_fail (filename != NULL, NULL);
 
   if (g_path_is_absolute (filename))
     return g_strdup (filename);
