@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -53,8 +53,9 @@
 #include "config.h"
 #endif
 
-#include "clutter-backend.h"
 #include "clutter-stage.h"
+
+#include "clutter-backend.h"
 #include "clutter-main.h"
 #include "clutter-color.h"
 #include "clutter-util.h"
@@ -78,12 +79,29 @@ G_DEFINE_TYPE (ClutterStage, clutter_stage, CLUTTER_TYPE_GROUP);
 #define CLUTTER_STAGE_GET_PRIVATE(obj) \
 (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_STAGE, ClutterStagePrivate))
 
+/* <private>
+ * ClutterStageHint:
+ * @CLUTTER_STAGE_NONE: No hint set
+ * @CLUTTER_STAGE_NO_CLEAR_ON_PAINT: When this hint is set, the stage
+ *   should not clear the viewport; this flag is useful when painting
+ *   fully opaque actors covering the whole visible area of the stage,
+ *   i.e. when no blending with the stage color happens over the whole
+ *   stage viewport
+ *
+ * A series of hints that enable or disable behaviours on the stage
+ */
+typedef enum { /*< prefix=CLUTTER_STAGE >*/
+  CLUTTER_STAGE_HINT_NONE = 0,
+
+  CLUTTER_STAGE_NO_CLEAR_ON_PAINT = 1 << 0
+} ClutterStageHint;
+
+#define STAGE_NO_CLEAR_ON_PAINT(s)      ((((ClutterStage *) (s))->priv->stage_hints & CLUTTER_STAGE_NO_CLEAR_ON_PAINT) != 0)
+
 struct _ClutterStagePrivate
 {
   /* the stage implementation */
   ClutterStageWindow *impl;
-  guint               minimum_width;
-  guint               minimum_height;
 
   ClutterColor        color;
   ClutterPerspective  perspective;
@@ -94,6 +112,8 @@ struct _ClutterStagePrivate
 
   GQueue             *event_queue;
 
+  ClutterStageHint    stage_hints;
+
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
   guint is_cursor_visible      : 1;
@@ -101,6 +121,7 @@ struct _ClutterStagePrivate
   guint use_fog                : 1;
   guint throttle_motion_events : 1;
   guint use_alpha              : 1;
+  guint min_size_changed       : 1;
 };
 
 enum
@@ -113,11 +134,12 @@ enum
   PROP_CURSOR_VISIBLE,
   PROP_PERSPECTIVE,
   PROP_TITLE,
-  PROP_USER_RESIZE,
+  PROP_USER_RESIZABLE,
   PROP_USE_FOG,
   PROP_FOG,
   PROP_USE_ALPHA,
-  PROP_KEY_FOCUS
+  PROP_KEY_FOCUS,
+  PROP_NO_CLEAR_HINT
 };
 
 enum
@@ -182,6 +204,7 @@ clutter_stage_allocate (ClutterActor           *self,
                         ClutterAllocationFlags  flags)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  ClutterGeometry geom = { 0, };
   gboolean origin_changed;
   gint width, height;
 
@@ -192,6 +215,11 @@ clutter_stage_allocate (ClutterActor           *self,
 
   width = clutter_actor_box_get_width (box);
   height = clutter_actor_box_get_height (box);
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
+
+  /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
+   * need to manually keep it informed of the current window size */
+  _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
 
   /* if the stage is fixed size (for instance, it's using a frame-buffer)
    * then we simply ignore any allocation request and override the
@@ -201,10 +229,6 @@ clutter_stage_allocate (ClutterActor           *self,
     {
       ClutterActorClass *klass;
 
-      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
-       * need to manually keep it informed of the current window size */
-      _cogl_onscreen_clutter_backend_set_size (width, height);
-
       CLUTTER_NOTE (LAYOUT,
                     "Following allocation to %dx%d (origin %s)",
                     width, height,
@@ -213,20 +237,43 @@ clutter_stage_allocate (ClutterActor           *self,
       klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
       klass->allocate (self, box, flags);
 
-      _clutter_stage_window_resize (priv->impl, width, height);
+      /* Ensure the window is sized correctly */
+      if (!priv->is_fullscreen)
+        {
+          if (priv->min_size_changed)
+            {
+              gfloat min_width, min_height;
+              gboolean min_width_set, min_height_set;
+
+              g_object_get (G_OBJECT (self),
+                            "min-width", &min_width,
+                            "min-width-set", &min_width_set,
+                            "min-height", &min_height,
+                            "min-height-set", &min_height_set,
+                            NULL);
+
+              if (!min_width_set)
+                min_width = 1;
+              if (!min_height_set)
+                min_height = 1;
+
+              if (width < min_width)
+                width = min_width;
+              if (height < min_height)
+                height = min_height;
+
+              priv->min_size_changed = FALSE;
+            }
+
+          if ((geom.width != width) || (geom.height != height))
+            _clutter_stage_window_resize (priv->impl, width, height);
+        }
     }
   else
     {
       ClutterActorBox override = { 0, };
-      ClutterGeometry geom = { 0, };
       ClutterActorClass *klass;
 
-      _clutter_stage_window_get_geometry (priv->impl, &geom);
-
-      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
-       * need to manually keep it informed of the current window size */
-      _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
-
       override.x1 = 0;
       override.y1 = 0;
       override.x2 = geom.width;
@@ -250,6 +297,7 @@ static void
 clutter_stage_paint (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  CoglBufferBit clear_flags;
   CoglColor stage_color;
   guint8 real_alpha;
   CLUTTER_STATIC_TIMER (stage_clear_timer,
@@ -276,10 +324,14 @@ clutter_stage_paint (ClutterActor *self)
                                            : 255);
   cogl_color_premultiply (&stage_color);
 
+  clear_flags = COGL_BUFFER_BIT_DEPTH;
+  if (!STAGE_NO_CLEAR_ON_PAINT (self))
+    clear_flags |= COGL_BUFFER_BIT_COLOR;
+
   CLUTTER_TIMER_START (_clutter_uprof_context, stage_clear_timer);
-  cogl_clear (&stage_color,
-	      COGL_BUFFER_BIT_COLOR |
-	      COGL_BUFFER_BIT_DEPTH);
+
+  cogl_clear (&stage_color, clear_flags);
+
   CLUTTER_TIMER_STOP (_clutter_uprof_context, stage_clear_timer);
 
   if (priv->use_fog)
@@ -324,7 +376,7 @@ clutter_stage_realize (ClutterActor *self)
    * first paint (which will likely occur before the ConfigureNotify
    * is received)
    */
-  CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_SYNC_MATRICES);
+  CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_SYNC_MATRICES);
 
   g_assert (priv->impl != NULL);
   is_realized = _clutter_stage_window_realize (priv->impl);
@@ -532,21 +584,36 @@ _clutter_stage_process_queued_events (ClutterStage *stage)
     {
       ClutterEvent *event;
       ClutterEvent *next_event;
+      ClutterInputDevice *device;
+      ClutterInputDevice *next_device;
+      gboolean check_device = FALSE;
 
       event = l->data;
       next_event = l->next ? l->next->data : NULL;
 
-      /* Skip consecutive motion events */
+      device = clutter_event_get_device (event);
+
+      if (next_event != NULL)
+        next_device = clutter_event_get_device (next_event);
+      else
+        next_device = NULL;
+
+      if (device != NULL && next_device != NULL)
+        check_device = TRUE;
+
+      /* Skip consecutive motion events coming from the same device */
       if (priv->throttle_motion_events &&
-          next_event &&
+          next_event != NULL &&
 	  event->type == CLUTTER_MOTION &&
 	  (next_event->type == CLUTTER_MOTION ||
-	   next_event->type == CLUTTER_LEAVE))
+	   next_event->type == CLUTTER_LEAVE) &&
+          (!check_device || (device == next_device)))
 	{
-	  CLUTTER_NOTE (EVENT,
-			"Omitting motion event at %.2f, %.2f",
-			event->motion.x, event->motion.y);
-	  goto next_event;
+          CLUTTER_NOTE (EVENT,
+                        "Omitting motion event at %d, %d",
+                        (int) event->motion.x,
+                        (int) event->motion.y);
+          goto next_event;
 	}
 
       _clutter_process_event (event);
@@ -631,6 +698,12 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
 {
   ClutterStage *stage = CLUTTER_STAGE (actor);
   ClutterStagePrivate *priv = stage->priv;
+  ClutterStageWindow *stage_window;
+  ClutterGeometry stage_clip;
+  const ClutterActorBox *clip;
+  ClutterActorBox bounds;
+  ClutterVertex v[4];
+  int i;
 
   CLUTTER_NOTE (PAINT, "Redraw request number %lu",
                 CLUTTER_CONTEXT ()->redraw_count + 1);
@@ -646,6 +719,67 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
     }
   else
     CLUTTER_CONTEXT ()->redraw_count += 1;
+
+  /* If the backend can't do anything with redraw clips (e.g. it already knows
+   * it needs to redraw everything anyway) then don't spend time transforming
+   * any clip regions into stage coordinates... */
+  stage_window = _clutter_stage_get_window (stage);
+  if (_clutter_stage_window_ignoring_redraw_clips (stage_window))
+    return;
+
+  /* Convert the clip rectangle (which is in leaf actor coordinates) into stage
+   * coordinates and then into an axis aligned stage coordinates bounding
+   * box...
+   */
+
+  clip = _clutter_actor_get_queue_redraw_clip (leaf);
+  if (!clip)
+    {
+      _clutter_stage_window_add_redraw_clip (stage_window, NULL);
+      return;
+    }
+
+  _clutter_actor_transform_and_project_box (leaf, clip, v);
+
+  bounds.x1 = v[0].x; bounds.y1 = v[0].y;
+  bounds.x2 = v[0].x; bounds.y2 = v[0].y;
+
+  for (i = 0; i < 4; i++)
+    {
+      if (v[i].x < bounds.x1)
+        bounds.x1 = v[i].x;
+      else if (v[i].x > bounds.x2)
+        bounds.x2 = v[i].x;
+
+      if (v[i].y < bounds.y1)
+        bounds.y1 = v[i].y;
+      else if (v[i].y > bounds.y2)
+        bounds.y2 = v[i].y;
+    }
+
+  /* when converting to integer coordinates make sure we round the edges of the
+   * clip rectangle outwards... */
+  stage_clip.x = bounds.x1;
+  stage_clip.y = bounds.y1;
+  stage_clip.width = ceilf (bounds.x2) - stage_clip.x;
+  stage_clip.height = ceilf (bounds.y2) - stage_clip.y;
+
+  _clutter_stage_window_add_redraw_clip (stage_window, &stage_clip);
+}
+
+gboolean
+_clutter_stage_has_full_redraw_queued (ClutterStage *stage)
+{
+  ClutterStageWindow *stage_window = _clutter_stage_get_window (stage);
+
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage) || stage_window == NULL)
+    return FALSE;
+
+  if (stage->priv->redraw_pending &&
+      !_clutter_stage_window_has_redraw_clips (stage_window))
+    return TRUE;
+  else
+    return FALSE;
 }
 
 static gboolean
@@ -666,13 +800,7 @@ clutter_stage_set_property (GObject      *object,
 			    const GValue *value,
 			    GParamSpec   *pspec)
 {
-  ClutterStage        *stage;
-  ClutterStagePrivate *priv;
-  ClutterActor        *actor;
-
-  stage = CLUTTER_STAGE (object);
-  actor = CLUTTER_ACTOR (stage);
-  priv = stage->priv;
+  ClutterStage *stage = CLUTTER_STAGE (object);
 
   switch (prop_id)
     {
@@ -700,7 +828,7 @@ clutter_stage_set_property (GObject      *object,
       clutter_stage_set_title (stage, g_value_get_string (value));
       break;
 
-    case PROP_USER_RESIZE:
+    case PROP_USER_RESIZABLE:
       clutter_stage_set_user_resizable (stage, g_value_get_boolean (value));
       break;
 
@@ -720,6 +848,10 @@ clutter_stage_set_property (GObject      *object,
       clutter_stage_set_key_focus (stage, g_value_get_object (value));
       break;
 
+    case PROP_NO_CLEAR_HINT:
+      clutter_stage_set_no_clear_hint (stage, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -760,7 +892,7 @@ clutter_stage_get_property (GObject    *gobject,
       g_value_set_string (value, priv->title);
       break;
 
-    case PROP_USER_RESIZE:
+    case PROP_USER_RESIZABLE:
       g_value_set_boolean (value, priv->is_user_resizable);
       break;
 
@@ -780,6 +912,15 @@ clutter_stage_get_property (GObject    *gobject,
       g_value_set_object (value, priv->key_focused_actor);
       break;
 
+    case PROP_NO_CLEAR_HINT:
+      {
+        gboolean hint =
+          (priv->stage_hints & CLUTTER_STAGE_NO_CLEAR_ON_PAINT) != 0;
+
+        g_value_set_boolean (value, hint);
+      }
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -877,8 +1018,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_boolean ("fullscreen-set",
-                                "Fullscreen Set",
-                                "Whether the main stage is fullscreen",
+                                P_("Fullscreen Set"),
+                                P_("Whether the main stage is fullscreen"),
                                 FALSE,
                                 CLUTTER_PARAM_READABLE);
   g_object_class_install_property (gobject_class,
@@ -894,9 +1035,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * and it might be deprecated at any later date.</para></warning>
    */
   pspec = g_param_spec_boolean ("offscreen",
-                                "Offscreen",
-                                "Whether the main stage should be "
-                                "rendered offscreen",
+                                P_("Offscreen"),
+                                P_("Whether the main stage should be rendered offscreen"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
@@ -908,9 +1048,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Whether the mouse pointer should be visible
    */
   pspec = g_param_spec_boolean ("cursor-visible",
-                                "Cursor Visible",
-                                "Whether the mouse pointer is visible "
-                                "on the main stage",
+                                P_("Cursor Visible"),
+                                P_("Whether the mouse pointer is visible on the main stage"),
                                 TRUE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
@@ -924,13 +1063,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_boolean ("user-resizable",
-                                "User Resizable",
-                                "Whether the stage is able to be resized "
-                                "via user interaction",
+                                P_("User Resizable"),
+                                P_("Whether the stage is able to be resized via user interaction"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
-                                   PROP_USER_RESIZE,
+                                   PROP_USER_RESIZABLE,
                                    pspec);
   /**
    * ClutterStage:color:
@@ -938,8 +1076,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * The color of the main stage.
    */
   pspec = clutter_param_spec_color ("color",
-                                    "Color",
-                                    "The color of the stage",
+                                    P_("Color"),
+                                    P_("The color of the stage"),
                                     &default_stage_color,
                                     CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_COLOR, pspec);
@@ -953,8 +1091,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.8.2
    */
   pspec = g_param_spec_boxed ("perspective",
-                              "Perspective",
-                              "Perspective projection parameters",
+                              P_("Perspective"),
+                              P_("Perspective projection parameters"),
                               CLUTTER_TYPE_PERSPECTIVE,
                               CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
@@ -969,8 +1107,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_string ("title",
-                               "Title",
-                               "Stage Title",
+                               P_("Title"),
+                               P_("Stage Title"),
                                NULL,
                                CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_TITLE, pspec);
@@ -985,8 +1123,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_boolean ("use-fog",
-                                "Use Fog",
-                                "Whether to enable depth cueing",
+                                P_("Use Fog"),
+                                P_("Whether to enable depth cueing"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_USE_FOG, pspec);
@@ -1000,8 +1138,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_boxed ("fog",
-                              "Fog",
-                              "Settings for the depth cueing",
+                              P_("Fog"),
+                              P_("Settings for the depth cueing"),
                               CLUTTER_TYPE_FOG,
                               CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_FOG, pspec);
@@ -1017,9 +1155,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_boolean ("use-alpha",
-                                "Use Alpha",
-                                "Whether to honour the alpha component of "
-                                "the stage color",
+                                P_("Use Alpha"),
+                                P_("Whether to honour the alpha component of the stage color"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_USE_ALPHA, pspec);
@@ -1035,12 +1172,29 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_object ("key-focus",
-                               "Key Focus",
-                               "The currently key focused actor",
+                               P_("Key Focus"),
+                               P_("The currently key focused actor"),
                                CLUTTER_TYPE_ACTOR,
                                CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_KEY_FOCUS, pspec);
 
+  /**
+   * ClutterStage:no-clear-hint:
+   *
+   * Whether or not the #ClutterStage should clear its contents
+   * before each paint cycle.
+   *
+   * See clutter_stage_set_no_clear_hint() for further information.
+   *
+   * Since: 1.4
+   */
+  pspec = g_param_spec_boolean ("no-clear-hint",
+                                P_("No Clear Hint"),
+                                P_("Whether the stage should clear its contents"),
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_NO_CLEAR_HINT, pspec);
+
   /**
    * ClutterStage::fullscreen
    * @stage: the stage which was fullscreened
@@ -1050,12 +1204,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.6
    */
   stage_signals[FULLSCREEN] =
-    g_signal_new ("fullscreen",
+    g_signal_new (I_("fullscreen"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterStageClass, fullscreen),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterStage::unfullscreen
@@ -1067,12 +1221,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.6
    */
   stage_signals[UNFULLSCREEN] =
-    g_signal_new ("unfullscreen",
+    g_signal_new (I_("unfullscreen"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterStageClass, unfullscreen),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterStage::activate
@@ -1084,12 +1238,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.6
    */
   stage_signals[ACTIVATE] =
-    g_signal_new ("activate",
+    g_signal_new (I_("activate"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterStageClass, activate),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterStage::deactivate
@@ -1101,12 +1255,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * Since: 0.6
    */
   stage_signals[DEACTIVATE] =
-    g_signal_new ("deactivate",
+    g_signal_new (I_("deactivate"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterStageClass, deactivate),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
   /**
@@ -1136,7 +1290,7 @@ clutter_stage_class_init (ClutterStageClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterStageClass, delete_event),
                   _clutter_boolean_handled_accumulator, NULL,
-                  clutter_marshal_BOOLEAN__BOXED,
+                  _clutter_marshal_BOOLEAN__BOXED,
                   G_TYPE_BOOLEAN, 1,
                   CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 
@@ -1148,6 +1302,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
   g_type_class_add_private (gobject_class, sizeof (ClutterStagePrivate));
 }
 
+static void
+clutter_stage_notify_min_size (ClutterStage *self)
+{
+  self->priv->min_size_changed = TRUE;
+}
+
 static void
 clutter_stage_init (ClutterStage *self)
 {
@@ -1155,7 +1315,7 @@ clutter_stage_init (ClutterStage *self)
   ClutterBackend *backend;
 
   /* a stage is a top-level object */
-  CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_IS_TOPLEVEL);
+  CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_IS_TOPLEVEL);
 
   self->priv = priv = CLUTTER_STAGE_GET_PRIVATE (self);
 
@@ -1179,10 +1339,9 @@ clutter_stage_init (ClutterStage *self)
   priv->is_cursor_visible      = TRUE;
   priv->use_fog                = FALSE;
   priv->throttle_motion_events = TRUE;
+  priv->min_size_changed       = FALSE;
 
   priv->color = default_stage_color;
-  priv->minimum_width = 1;
-  priv->minimum_height = 1;
 
   priv->perspective.fovy   = 60.0; /* 60 Degrees */
   priv->perspective.aspect = 1.0;
@@ -1196,6 +1355,11 @@ clutter_stage_init (ClutterStage *self)
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
   clutter_stage_set_title (self, g_get_prgname ());
   clutter_stage_set_key_focus (self, NULL);
+
+  g_signal_connect (self, "notify::min-width",
+                    G_CALLBACK (clutter_stage_notify_min_size), NULL);
+  g_signal_connect (self, "notify::min-height",
+                    G_CALLBACK (clutter_stage_notify_min_size), NULL);
 }
 
 /**
@@ -1225,6 +1389,7 @@ clutter_stage_get_default (void)
        * reference will be claimed by the stage manager.
        */
       stage = g_object_new (CLUTTER_TYPE_STAGE, NULL);
+      _clutter_stage_manager_set_default_stage (stage_manager, stage);
 
       /* the default stage is realized by default */
       clutter_actor_realize (CLUTTER_ACTOR (stage));
@@ -1253,8 +1418,7 @@ clutter_stage_set_color (ClutterStage       *stage,
 
   priv->color = *color;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (stage))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 
   g_object_notify (G_OBJECT (stage), "color");
 }
@@ -1304,7 +1468,7 @@ clutter_stage_set_perspective (ClutterStage       *stage,
   /* this will cause the viewport to be reset; see
    * clutter_maybe_setup_viewport() inside clutter-main.c
    */
-  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
+  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_SYNC_MATRICES);
 }
 
 /**
@@ -1880,8 +2044,7 @@ clutter_stage_set_use_fog (ClutterStage *stage,
       CLUTTER_NOTE (MISC, "%s depth-cueing inside stage",
                     priv->use_fog ? "enabling" : "disabling");
 
-      if (CLUTTER_ACTOR_IS_VISIBLE (stage))
-        clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+      clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 
       g_object_notify (G_OBJECT (stage), "use-fog");
     }
@@ -1958,7 +2121,7 @@ clutter_stage_set_fog (ClutterStage *stage,
 
   priv->fog = *fog;
 
-  if (priv->use_fog && CLUTTER_ACTOR_IS_VISIBLE (stage))
+  if (priv->use_fog)
     clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 }
 
@@ -2117,7 +2280,7 @@ clutter_stage_ensure_viewport (ClutterStage *stage)
 {
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
-  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
+  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_SYNC_MATRICES);
 
   clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 }
@@ -2333,8 +2496,16 @@ clutter_stage_get_use_alpha (ClutterStage *stage)
  * @width: width, in pixels
  * @height: height, in pixels
  *
- * Sets the minimum size for a stage window. This has no effect if the stage
- * is fullscreen.
+ * Sets the minimum size for a stage window, if the default backend
+ * uses #ClutterStage inside a window
+ *
+ * This is a convenience function, and it is equivalent to setting the
+ * #ClutterActor:min-width and #ClutterActor:min-height on @stage
+ *
+ * If the current size of @stage is smaller than the minimum size, the
+ * @stage will be resized to the new @width and @height
+ *
+ * This function has no effect if @stage is fullscreen
  *
  * Since: 1.2
  */
@@ -2343,57 +2514,143 @@ clutter_stage_set_minimum_size (ClutterStage *stage,
                                 guint         width,
                                 guint         height)
 {
-  gboolean resize;
-  ClutterGeometry geom;
-
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
   g_return_if_fail ((width > 0) && (height > 0));
 
-  stage->priv->minimum_width = width;
-  stage->priv->minimum_height = height;
-
-  if (stage->priv->impl == NULL)
-    return;
-
-  resize = FALSE;
-  _clutter_stage_window_get_geometry (stage->priv->impl, &geom);
-
-  if (geom.width < width)
-    resize = TRUE;
-  else
-    width = geom.width;
-
-  if (geom.height < height)
-    resize = TRUE;
-  else
-    height = geom.height;
-
-  if (resize)
-    _clutter_stage_window_resize (stage->priv->impl, width, height);
+  g_object_set (G_OBJECT (stage),
+                "min-width", (gfloat) width,
+                "min-height", (gfloat )height,
+                NULL);
 }
 
 /**
  * clutter_stage_get_minimum_size:
  * @stage: a #ClutterStage
- * @width: width, in pixels
- * @height: height, in pixels
+ * @width: (out): return location for the minimum width, in pixels,
+ *   or %NULL
+ * @height: (out): return location for the minimum height, in pixels,
+ *   or %NULL
  *
- * Gets the set minimum size for a stage window. This may not correspond
- * to the actual minimum size and is specific to the back-end
- * implementation.
+ * Retrieves the minimum size for a stage window as set using
+ * clutter_stage_set_minimum_size().
+ *
+ * The returned size may not correspond to the actual minimum size and
+ * it is specific to the #ClutterStage implementation inside the
+ * Clutter backend
  *
  * Since: 1.2
  */
 void
 clutter_stage_get_minimum_size (ClutterStage *stage,
-                                guint        *width,
-                                guint        *height)
+                                guint        *width_p,
+                                guint        *height_p)
 {
+  gfloat width, height;
+  gboolean width_set, height_set;
+
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
-  if (width)
-    *width = stage->priv->minimum_width;
-  if (height)
-    *height = stage->priv->minimum_height;
+  g_object_get (G_OBJECT (stage),
+                "min-width", &width,
+                "min-width-set", &width_set,
+                "min-height", &height,
+                "min-height-set", &height_set,
+                NULL);
+
+  /* if not width or height have been set, then the Stage
+   * minimum size is defined to be 1x1
+   */
+  if (!width_set)
+    width = 1;
+
+  if (!height_set)
+    height = 1;
+
+  if (width_p)
+    *width_p = (guint) width;
+
+  if (height_p)
+    *height_p = (guint) height;
 }
 
+/* Returns the number of swap buffers pending completion for the stage */
+int
+_clutter_stage_get_pending_swaps (ClutterStage *stage)
+{
+  ClutterStageWindow *stage_window;
+
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
+    return 0;
+
+  stage_window = _clutter_stage_get_window (stage);
+  if (stage_window == NULL)
+    return 0;
+
+  return _clutter_stage_window_get_pending_swaps (stage_window);
+}
+
+/**
+ * clutter_stage_set_no_clear_hint:
+ * @stage: a #ClutterStage
+ * @no_clear: %TRUE if the @stage should not clear itself on every
+ *   repaint cycle
+ *
+ * Sets whether the @stage should clear itself at the beginning
+ * of each paint cycle or not.
+ *
+ * Clearing the #ClutterStage can be a costly operation, especially
+ * if the stage is always covered - for instance, in a full-screen
+ * video player or in a game with a background texture.
+ *
+ * <note><para>This setting is a hint; Clutter might discard this
+ * hint depending on its internal state.</para></note>
+ *
+ * <warning><para>If parts of the stage are visible and you disable
+ * clearing you might end up with visual artifacts while painting the
+ * contents of the stage.</para></warning>
+ *
+ * Since: 1.4
+ */
+void
+clutter_stage_set_no_clear_hint (ClutterStage *stage,
+                                 gboolean      no_clear)
+{
+  ClutterStagePrivate *priv;
+  ClutterStageHint new_hints;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+  new_hints = priv->stage_hints;
+
+  if (no_clear)
+    new_hints |= CLUTTER_STAGE_NO_CLEAR_ON_PAINT;
+  else
+    new_hints &= ~CLUTTER_STAGE_NO_CLEAR_ON_PAINT;
+
+  if (priv->stage_hints == new_hints)
+    return;
+
+  priv->stage_hints = new_hints;
+
+  g_object_notify (G_OBJECT (stage), "no-clear-hint");
+}
+
+/**
+ * clutter_stage_get_no_clear_hint:
+ * @stage: a #ClutterStage
+ *
+ * Retrieves the hint set with clutter_stage_set_no_clear_hint()
+ *
+ * Return value: %TRUE if the stage should not clear itself on every paint
+ *   cycle, and %FALSE otherwise
+ *
+ * Since: 1.4
+ */
+gboolean
+clutter_stage_get_no_clear_hint (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  return (stage->priv->stage_hints & CLUTTER_STAGE_NO_CLEAR_ON_PAINT) != 0;
+}
