@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,12 +55,11 @@ _cogl_bitmap_can_premult (CoglPixelFormat format)
   return FALSE;
 }
 
-gboolean
-_cogl_bitmap_convert (const CoglBitmap *bmp,
-		      CoglBitmap       *dst_bmp,
+CoglBitmap *
+_cogl_bitmap_convert (CoglBitmap *bmp,
 		      CoglPixelFormat   dst_format)
 {
-  return FALSE;
+  return NULL;
 }
 
 gboolean
@@ -77,22 +76,6 @@ _cogl_bitmap_premult (CoglBitmap *dst_bmp)
 
 #ifdef USE_QUARTZ
 
-/* lacking GdkPixbuf and other useful GError domains, define one of our own */
-
-#define COGL_BITMAP_ERROR cogl_bitmap_error_quark ()
-
-typedef enum {
-  COGL_BITMAP_ERROR_FAILED,
-  COGL_BITMAP_ERROR_UNKNOWN_TYPE,
-  COGL_BITMAP_ERROR_CORRUPT_IMAGE
-} CoglBitmapError;
-
-GQuark
-cogl_bitmap_error_quark (void)
-{
-  return g_quark_from_static_string ("cogl-bitmap-error-quark");
-}
-
 gboolean
 _cogl_bitmap_get_size_from_file (const char *filename,
                                  int        *width,
@@ -108,12 +91,10 @@ _cogl_bitmap_get_size_from_file (const char *filename,
 }
 
 /* the error does not contain the filename as the caller already has it */
-gboolean
-_cogl_bitmap_from_file (CoglBitmap  *bmp,
-			const char  *filename,
+CoglBitmap *
+_cogl_bitmap_from_file (const char  *filename,
 			GError     **error)
 {
-  g_assert (bmp != NULL);
   g_assert (filename != NULL);
   g_assert (error == NULL || *error == NULL);
 
@@ -126,7 +107,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
       /* doesn't exist, not readable, etc. */
       g_set_error (error, COGL_BITMAP_ERROR, COGL_BITMAP_ERROR_FAILED,
                    "%s", g_strerror (save_errno));
-      return FALSE;
+      return NULL;
     }
 
   /* Unknown images would be cleanly caught as zero width/height below, but try
@@ -138,7 +119,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
       CFRelease (image_source);
       g_set_error (error, COGL_BITMAP_ERROR, COGL_BITMAP_ERROR_UNKNOWN_TYPE,
                    "Unknown image type");
-      return FALSE;
+      return NULL;
     }
   CFRelease (type);
 
@@ -153,7 +134,7 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
       CFRelease (image);
       g_set_error (error, COGL_BITMAP_ERROR, COGL_BITMAP_ERROR_CORRUPT_IMAGE,
                    "Image has zero width or height");
-      return FALSE;
+      return NULL;
     }
 
   /* allocate buffer big enough to hold pixel data */
@@ -176,13 +157,12 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
   CGContextRelease (bitmap_context);
 
   /* store bitmap info */
-  bmp->data = out_data;
-  bmp->format = COGL_PIXEL_FORMAT_ARGB_8888;
-  bmp->width = width;
-  bmp->height = height;
-  bmp->rowstride = rowstride;
-
-  return TRUE;
+  return _cogl_bitmap_new_from_data (out_data,
+                                     COGL_PIXEL_FORMAT_ARGB_8888,
+                                     width, height,
+                                     rowstride,
+                                     (CoglBitmapDestroyNotify) g_free,
+                                     NULL);
 }
 
 #elif defined(USE_GDKPIXBUF)
@@ -200,9 +180,15 @@ _cogl_bitmap_get_size_from_file (const char *filename,
   return FALSE;
 }
 
-gboolean
-_cogl_bitmap_from_file (CoglBitmap   *bmp,
-			const char   *filename,
+static void
+_cogl_bitmap_unref_pixbuf (guint8 *pixels,
+                           void *pixbuf)
+{
+  g_object_unref (pixbuf);
+}
+
+CoglBitmap *
+_cogl_bitmap_from_file (const char   *filename,
 			GError      **error)
 {
   GdkPixbuf        *pixbuf;
@@ -212,9 +198,9 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   int               width;
   int               height;
   int               rowstride;
+  int               aligned_rowstride;
   int               bits_per_sample;
   int               n_channels;
-  int               last_row_size;
   guint8           *pixels;
   guint8           *out_data;
   guint8           *out;
@@ -222,9 +208,6 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
 
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  if (bmp == NULL)
-    return FALSE;
-
   /* Load from file using GdkPixbuf */
   pixbuf = gdk_pixbuf_new_from_file (filename, error);
   if (pixbuf == NULL)
@@ -239,9 +222,6 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
   bits_per_sample = gdk_pixbuf_get_bits_per_sample (pixbuf);
   n_channels      = gdk_pixbuf_get_n_channels (pixbuf);
 
-  /* The docs say this is the right way */
-  last_row_size = width * ((n_channels * bits_per_sample + 7) / 8);
-
   /* According to current docs this should be true and so
    * the translation to cogl pixel format below valid */
   g_assert (bits_per_sample == 8);
@@ -267,37 +247,55 @@ _cogl_bitmap_from_file (CoglBitmap   *bmp,
       return FALSE;
     }
 
-  /* FIXME: Any way to destroy pixbuf but retain pixel data? */
+  /* Work out what the rowstride would be if it was packing the data
+     but aligned to 4 bytes */
+  aligned_rowstride = (width * n_channels + 3) & ~3;
+
+  /* The documentation for GdkPixbuf states that is not safe to read
+     all of the data as height*rowstride because the buffer might not
+     be allocated to include the full length of the rowstride for the
+     last row so arguably we should always copy the buffer when
+     rowstride != width*bpp because some places in Cogl assume that it
+     can memcpy(height*rowstride). However that rule is probably only
+     in place so that GdkPixbuf can implement gdk_pixbuf_new_subpixbuf
+     by just sharing the data and setting a large rowstride. That does
+     not apply in this case because we are just creating a new buffer
+     for a file. It seems very unlikely that GdkPixbuf would not
+     allocate the full rowstride in this case and it is highly
+     desirable to avoid copying the buffer. This instead just assumes
+     that whatever buffer pixbuf points into will always be allocated
+     to a 4-byte aligned buffer so we can avoid copying unless the
+     rowstride is unusually large */
+  if (rowstride <= aligned_rowstride)
+    return _cogl_bitmap_new_from_data (gdk_pixbuf_get_pixels (pixbuf),
+                                       pixel_format,
+                                       width,
+                                       height,
+                                       rowstride,
+                                       _cogl_bitmap_unref_pixbuf,
+                                       pixbuf);
 
   pixels   = gdk_pixbuf_get_pixels (pixbuf);
-  out_data = g_malloc (height * rowstride);
+  out_data = g_malloc (aligned_rowstride * height);
   out      = out_data;
 
-  /* Copy up to last row */
-  for (r = 0; r < height-1; ++r)
+  for (r = 0; r < height; ++r)
     {
-      memcpy (out, pixels, rowstride);
+      memcpy (out, pixels, n_channels * width);
       pixels += rowstride;
-      out += rowstride;
+      out += aligned_rowstride;
     }
 
-  /* Copy last row */
-  memcpy (out, pixels, last_row_size);
-
   /* Destroy GdkPixbuf object */
   g_object_unref (pixbuf);
 
-  /* Store bitmap info */
-  bmp->data = out_data; /* The stored data the same alignment constraints as a
-                         * gdkpixbuf but stores a full rowstride in the last
-                         * scanline
-                         */
-  bmp->format = pixel_format;
-  bmp->width = width;
-  bmp->height = height;
-  bmp->rowstride = rowstride;
-
-  return TRUE;
+  return _cogl_bitmap_new_from_data (out_data,
+                                     pixel_format,
+                                     width,
+                                     height,
+                                     aligned_rowstride,
+                                     (CoglBitmapDestroyNotify) g_free,
+                                     NULL);
 }
 
 #else
@@ -318,11 +316,11 @@ _cogl_bitmap_get_size_from_file (const char *filename,
   return TRUE;
 }
 
-gboolean
-_cogl_bitmap_from_file (CoglBitmap  *bmp,
-			const char  *filename,
+CoglBitmap *
+_cogl_bitmap_from_file (const char  *filename,
 			GError     **error)
 {
+  CoglBitmap *bmp;
   int      stb_pixel_format;
   int      width;
   int      height;
@@ -330,9 +328,6 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
 
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  if (bmp == NULL)
-    return FALSE;
-
   /* Load from file using stb */
   pixels = stbi_load (filename,
                       &width, &height, &stb_pixel_format,
@@ -341,14 +336,15 @@ _cogl_bitmap_from_file (CoglBitmap  *bmp,
     return FALSE;
 
   /* Store bitmap info */
-  bmp->data = g_memdup (pixels, height * width * 4);
-  bmp->format = COGL_PIXEL_FORMAT_RGBA_8888;
-  bmp->width = width;
-  bmp->height = height;
-  bmp->rowstride = width * 4;
+  bmp = _cogl_bitmap_new_from_data (g_memdup (pixels, height * width * 4),
+                                    COGL_PIXEL_FORMAT_RGBA_8888,
+                                    width, height,
+                                    width * 4,
+                                    (CoglBitmapDestroyNotify) g_free,
+                                    NULL);
 
   free (pixels);
 
-  return TRUE;
+  return bmp;
 }
 #endif
