@@ -32,6 +32,7 @@
 #include "cogl-journal-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
+#include "cogl-material-opengl-private.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-path-private.h"
 
@@ -214,7 +215,6 @@ cogl_create_context (void)
   _cogl_flush_face_winding ();
 
   _context->atlas = NULL;
-  _context->atlas_texture = COGL_INVALID_HANDLE;
 
   /* As far as I can tell, GL_POINT_SPRITE doesn't have any effect
      unless GL_COORD_REPLACE is enabled for an individual
@@ -274,8 +274,6 @@ _cogl_destroy_context (void)
 
   if (_context->atlas)
     _cogl_atlas_free (_context->atlas);
-  if (_context->atlas_texture)
-    cogl_handle_unref (_context->atlas_texture);
 
   _cogl_bitmask_destroy (&_context->texcoord_arrays_enabled);
   _cogl_bitmask_destroy (&_context->temp_bitmask);
