@@ -226,6 +226,7 @@ typedef enum { /*< prefix=COGL_PIXEL_FORMAT >*/
  * @COGL_FEATURE_TEXTURE_YUV: ycbcr conversion support
  * @COGL_FEATURE_TEXTURE_READ_PIXELS: glReadPixels() support
  * @COGL_FEATURE_SHADERS_GLSL: GLSL support
+ * @COGL_FEATURE_SHADERS_ARBFP: ARBFP support
  * @COGL_FEATURE_OFFSCREEN: FBO support
  * @COGL_FEATURE_OFFSCREEN_MULTISAMPLE: Multisample support on FBOs
  * @COGL_FEATURE_OFFSCREEN_BLIT: Blit support on FBOs
@@ -250,6 +251,7 @@ typedef enum { /*< prefix=COGL_PIXEL_FORMAT >*/
  *     hardware.
  * @COGL_FEATURE_POINT_SPRITE: Whether
  *     cogl_material_set_layer_point_sprite_coords_enabled() is supported.
+ * @COGL_FEATURE_TEXTURE_3D: 3D texture support
  *
  * Flags for the supported features.
  *
@@ -274,7 +276,9 @@ typedef enum
   COGL_FEATURE_TEXTURE_NPOT_BASIC     = (1 << 15),
   COGL_FEATURE_TEXTURE_NPOT_MIPMAP    = (1 << 16),
   COGL_FEATURE_TEXTURE_NPOT_REPEAT    = (1 << 17),
-  COGL_FEATURE_POINT_SPRITE           = (1 << 18)
+  COGL_FEATURE_POINT_SPRITE           = (1 << 18),
+  COGL_FEATURE_TEXTURE_3D             = (1 << 19),
+  COGL_FEATURE_SHADERS_ARBFP          = (1 << 20)
 } CoglFeatureFlags;
 
 /**
@@ -429,18 +433,33 @@ cogl_blend_string_error_quark (void);
 
 /**
  * CoglError:
- * @COGL_ERROR_MISSING_FEATURE: You tried to use a feature not
- *    currently available; likely because of missing driver support.
+ * @COGL_ERROR_UNSUPPORTED: You tried to use a feature or
+ *    configuration not currently available.
  *
  * Error enumeration for Cogl
  *
+ * The @COGL_ERROR_UNSUPPORTED error can be thrown for a variety of
+ * reasons. For example:
+ *
+ * <itemizedlist>
+ *  <listitem><para>You've tried to use a feature that is not
+ *   advertised by cogl_get_features(). This could happen if you create
+ *   a non-sliced texture with a non-power-of-two size when
+ *   %COGL_FEATURE_TEXTURE_NPOT is not advertised.</para></listitem>
+ *  <listitem><para>The GPU can not handle the configuration you have
+ *   requested. An example might be if you try to use too many texture
+ *   layers in a single #CoglMaterial</para></listitem>
+ *  <listitem><para>The driver does not support some
+ *   configuration.</para></listiem>
+ * </itemizedlist>
+ *
  * Currently this is only used by Cogl API marked as experimental so
  * this enum should also be considered experimental.
  *
  * Since: 1.4
  */
 typedef enum { /*< prefix=COGL_ERROR >*/
-  COGL_ERROR_MISSING_FEATURE
+  COGL_ERROR_UNSUPPORTED
 } CoglError;
 
 GQuark
