@@ -30,6 +30,7 @@
 #endif
 
 #include "cogl.h"
+#include "cogl-debug.h"
 #include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-texture-pixmap-x11.h"
@@ -42,6 +43,7 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-xlib.h"
+#include "cogl-material-opengl-private.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -1286,7 +1288,7 @@ static void
 _cogl_texture_pixmap_x11_set_wrap_mode_parameters (CoglTexture *tex,
                                                    GLenum       wrap_mode_s,
                                                    GLenum       wrap_mode_t,
-                                                   GLenum       wrap_mode_r)
+                                                   GLenum       wrap_mode_p)
 {
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
   CoglHandle child_tex;
@@ -1297,7 +1299,7 @@ _cogl_texture_pixmap_x11_set_wrap_mode_parameters (CoglTexture *tex,
   _cogl_texture_set_wrap_mode_parameters (child_tex,
                                           wrap_mode_s,
                                           wrap_mode_t,
-                                          wrap_mode_r);
+                                          wrap_mode_p);
 }
 
 static CoglPixelFormat
